@@ -22,9 +22,9 @@ import (
 	p "github.com/kr/pretty"
 	api "github.com/osrg/gobgp/api"
 	"github.com/osrg/gobgp/config"
-	ops "github.com/osrg/gobgp/openswitch"
 	"github.com/osrg/gobgp/packet/bgp"
 	"github.com/osrg/gobgp/server"
+	"github.com/osrg/gobgp/table"
 	"io/ioutil"
 	"log/syslog"
 	"net/http"
@@ -50,7 +50,6 @@ func main() {
 		Facility        string `long:"syslog-facility" description:"specify syslog facility"`
 		DisableStdlog   bool   `long:"disable-stdlog" description:"disable standard logging"`
 		CPUs            int    `long:"cpus" description:"specify the number of CPUs to be used"`
-		Ops             bool   `long:"openswitch" description:"openswitch mode"`
 		GrpcHosts       string `long:"api-hosts" description:"specify the hosts that gobgpd listens on" default:":50051"`
 		GracefulRestart bool   `short:"r" long:"graceful-restart" description:"flag restart-state in graceful-restart capability"`
 		Dry             bool   `short:"d" long:"dry-run" description:"check configuration"`
@@ -187,15 +186,7 @@ func main() {
 		}
 	}()
 
-	if opts.Ops {
-		m, err := ops.NewOpsManager(opts.GrpcHosts)
-		if err != nil {
-			log.Errorf("Failed to start ops config manager: %s", err)
-			os.Exit(1)
-		}
-		log.Info("Coordination with OpenSwitch")
-		m.Serve()
-	} else if opts.ConfigFile != "" {
+	if opts.ConfigFile != "" {
 		go config.ReadConfigfileServe(opts.ConfigFile, opts.ConfigType, configCh)
 	}
 
@@ -260,20 +251,61 @@ func main() {
 					p := config.ConfigSetToRoutingPolicy(newConfig)
 					bgpServer.UpdatePolicy(*p)
 				}
+				// global policy update
+				if !newConfig.Global.ApplyPolicy.Config.Equal(&c.Global.ApplyPolicy.Config) {
+					a := newConfig.Global.ApplyPolicy.Config
+					toDefaultTable := func(r config.DefaultPolicyType) table.RouteType {
+						var def table.RouteType
+						switch r {
+						case config.DEFAULT_POLICY_TYPE_ACCEPT_ROUTE:
+							def = table.ROUTE_TYPE_ACCEPT
+						case config.DEFAULT_POLICY_TYPE_REJECT_ROUTE:
+							def = table.ROUTE_TYPE_REJECT
+						}
+						return def
+					}
+					toPolicyDefinitions := func(r []string) []*config.PolicyDefinition {
+						p := make([]*config.PolicyDefinition, 0, len(r))
+						for _, n := range r {
+							p = append(p, &config.PolicyDefinition{
+								Name: n,
+							})
+						}
+						return p
+					}
+
+					def := toDefaultTable(a.DefaultImportPolicy)
+					ps := toPolicyDefinitions(a.ImportPolicyList)
+					bgpServer.ReplacePolicyAssignment("", table.POLICY_DIRECTION_IMPORT, ps, def)
+
+					def = toDefaultTable(a.DefaultExportPolicy)
+					ps = toPolicyDefinitions(a.ExportPolicyList)
+					bgpServer.ReplacePolicyAssignment("", table.POLICY_DIRECTION_EXPORT, ps, def)
+
+					updatePolicy = true
+
+				}
 				c = newConfig
 			}
 
 			for i, p := range added {
 				log.Infof("Peer %v is added", p.Config.NeighborAddress)
-				bgpServer.AddNeighbor(&added[i])
+				if err := bgpServer.AddNeighbor(&added[i]); err != nil {
+					log.Warn(err)
+				}
 			}
 			for i, p := range deleted {
 				log.Infof("Peer %v is deleted", p.Config.NeighborAddress)
-				bgpServer.DeleteNeighbor(&deleted[i])
+				if err := bgpServer.DeleteNeighbor(&deleted[i]); err != nil {
+					log.Warn(err)
+				}
 			}
 			for i, p := range updated {
 				log.Infof("Peer %v is updated", p.Config.NeighborAddress)
-				u, _ := bgpServer.UpdateNeighbor(&updated[i])
+				u, err := bgpServer.UpdateNeighbor(&updated[i])
+				if err != nil {
+					log.Warn(err)
+				}
 				updatePolicy = updatePolicy || u
 			}
 
