@@ -26,7 +26,7 @@ func NewTestBGPOpenMessage() *BGPMessage {
 		[]ParameterCapabilityInterface{NewCapMultiProtocol(RF_IPv4_UC)})
 	g := &CapGracefulRestartTuple{4, 2, 3}
 	p3 := NewOptionParameterCapability(
-		[]ParameterCapabilityInterface{NewCapGracefulRestart(false, 100,
+		[]ParameterCapabilityInterface{NewCapGracefulRestart(false, true, 100,
 			[]*CapGracefulRestartTuple{g})})
 	p4 := NewOptionParameterCapability(
 		[]ParameterCapabilityInterface{NewCapFourOctetASNumber(100000)})
